////////////////////////////////////////////////////////////
// functions for computing sine, cosine and winding number 
// sums of angles. Inverse functions.
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// Compute quadrant of angle given sine and cosine.
////////////////////////////////////////////////////////////
region:=function(c1,q1,s1,e1);

tc:=quadptest(c1,q1);
ts:=quadptest(s1,e1);

if tc gt 0 and ts ge 0 then return 1; end if;
if tc le 0 and ts gt 0 then return 2; end if;
if tc lt 0 and ts le 0 then return 3; end if;
if tc ge 0 and ts lt 0 then return 4; end if;

print "error in region, conditions not satisfied"; return -1; 

end function;//of region;
////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////
// Given sine(theta_i) and cosine(theta_i), i=1,2 compute
// sine(theta_1+theta_2),cosine(theta_1+theta_2) and the
// winding number. The sines and cosines are assumed to 
// be in Q(sqrt(2),sqrt(3),sqrt(5),...). The form
// of these elements is x,y where x=[x1,x2,x3,...,xn] 
// and y=[y1,y2,....,yn] which represents x1*sqrt(y1) + x2*sqrt(y2) + ....
// xn*sqrt(yn)
// with yi > 0, i=1,..,n.
////////////////////////////////////////////////////////////
angle_add:=function(c1,q1,s1,e1,c2,q2,s2,e2);

c3,q3:=quadmult(c1,q1,c1,q1);
s3,e3:=quadmult(s1,e1,s1,e1);
a1,b1:=quadadd(c3,q3,s3,e3);
if [a1,b1] ne [[1],[1]] then
 print "error in angle_add cos^2(angle1) + sin^2(angle1) ne 1";
 return -1;
end if;

c3,q3:=quadmult(c2,q2,c2,q2);
s3,e3:=quadmult(s2,e2,s2,e2);
a1,b1:=quadadd(c3,q3,s3,e3);
if [a1,b1] ne [[1],[1]] then
 print "error in angle_add cos^2(angle2) + sin^2(angle2) ne 1";
 return -1;
end if;


c3,q3:=quadmult(s1,e1,s2,e2);
c3,q3:=quadmult([-1],[1],c3,q3);
c4,q4:=quadmult(c1,q1,c2,q2);
c3,q3:=quadadd(c4,q4,c3,q3);

s3,e3:=quadmult(s1,e1,c2,q2);
s4,e4:=quadmult(c1,q1,s2,e2);
s3,e3:=quadadd(s3,e3,s4,e4);


r1:=region(c1,q1,s1,e1);
r2:=region(c2,q2,s2,e2);
r3:=region(c3,q3,s3,e3);

if r1+r2-1 le r3 then r:=0; else r:=1; end if;

return c3,q3,s3,e3,r;


end function;//of angle_add;
////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////
// Compute an angle give the sine and cosine. Only angles
// which can occur are computed, otherwise return -1.
////////////////////////////////////////////////////////////
arc_angle:=function(c2,q2,s2,e2);

c3,q3:=quadmult(c2,q2,c2,q2);
s3,e3:=quadmult(s2,e2,s2,e2);
a1,b1:=quadadd(c3,q3,s3,e3);
if [a1,b1] ne [[1],[1]] then
 print "error in arc_angle cos^2(angle) + sin^2(angle) ne 1";
 return -1;
end if;

if #c2 ne 1 or #q2 ne 1 or #s2 ne 1 or #e2 ne 1 then 
 print "error in arc_angle, cannot find angle";
 return -1;
end if;

c1:=c2[1];
q1:=q2[1];
s1:=s2[1];
e1:=e2[1];

if s1 eq 0   and e1 eq 1 and c1 eq 1   and q1 eq 1 then return 0  ; end if;
if s1 eq 1/2 and e1 eq 1 and c1 eq 1/2 and q1 eq 3 then return 1/6; end if;
if s1 eq 1/2 and e1 eq 2 and c1 eq 1/2 and q1 eq 2 then return 1/4; end if;
if s1 eq 1/2 and e1 eq 3 and c1 eq 1/2 and q1 eq 1 then return 1/3; end if;
if s1 eq 1   and e1 eq 1 and c1 eq 0   and q1 eq 1 then return 1/2; end if;

if s1 eq 1/2 and e1 eq 3 and c1 eq -1/2   and q1 eq 1 then return 2/3; end if;
if s1 eq 1/2 and e1 eq 2 and c1 eq -1/2 and q1 eq 2 then return 3/4; end if;
if s1 eq 1/2 and e1 eq 1 and c1 eq -1/2 and q1 eq 3 then return 5/6; end if;
if s1 eq 0   and e1 eq 1 and c1 eq -1   and q1 eq 1 then return 1  ; end if;

if s1 eq -1/2 and e1 eq 1 and c1 eq -1/2 and q1 eq 3 then return 7/6; end if;
if s1 eq -1/2 and e1 eq 2 and c1 eq -1/2 and q1 eq 2 then return 5/4; end if;
if s1 eq -1/2 and e1 eq 3 and c1 eq -1/2 and q1 eq 1 then return 4/3; end if;
if s1 eq -1   and e1 eq 1 and c1 eq 0   and q1 eq 1 then return 3/2; end if;

if s1 eq -1/2 and e1 eq 3 and c1 eq 1/2   and q1 eq 1 then return 5/3; end if;
if s1 eq -1/2 and e1 eq 2 and c1 eq 1/2 and q1 eq 2 then return 7/4; end if;
if s1 eq -1/2 and e1 eq 1 and c1 eq 1/2 and q1 eq 3 then return 11/6; end if;


print "angle not found in arc_angle";
return -1;

end function;//of arc_angle
////////////////////////////////////////////////////////////










