////////////////////////////////////////////////////////////
// functions for computing hyperbolic area.
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// area computes the hyperbolic area of
// region bound between the vertical
// lines x=u1 and x=u1
// and the circles of refined_cover
// A factor of Pi has been removed
// to make the area rational.
////////////////////////////////////////////////////////////
area:=function(u1,u2,refined_cover,g);

corrected_cover := correct_cover(u1,u2,refined_cover,g);
//if covered cannot be corrected, then area fails.
if corrected_cover eq [ [ ] ] then 
 return -1;
end if;

k:=#corrected_cover;
area:=k;

cir1:=corrected_cover[1];
cirk:=corrected_cover[k];

r:=0;
c1,q1,s1,e1:=line_angle(cir1,u1,1);

pairs := pairing(u1,u2,corrected_cover,g);
v:=vorbits(pairs);

if v eq [[ ]] then return -1; end if;

for j:=1 to #v do
 for i in v[j] do
  if (i ne 1) and (i ne k+1) then
   c2,q2,s2,e2:=int_angle(corrected_cover[i-1],corrected_cover[i]);
   c1,q1,s1,e1,r1:=angle_add(c1,q1,s1,e1,c2,q2,s2,e2);
   r:=r+r1;
  end if;
 end for;
end for;

c2,q2,s2,e2:=line_angle(cirk,u2,-1);
c1,q1,s1,e1,r1:=angle_add(c1,q1,s1,e1,c2,q2,s2,e2);
r:=r+r1;


ac:=arc_angle(c1,q1,s1,e1);
if ac eq -1 then 
 return -1;
else
 area:=area -2*r-ac;
end if;


return area;

end function;//of area
////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////
// computes area of fundamental region
// of Gamma_0(f)^+ for squarefree f
// using index of Gamma_0(f) in Gamma(1)
// and Gamma_0(f)^+
////////////////////////////////////////////////////////////
area_ind:=function(n);
pbn:=PrimeBasis(n);
ind:=n;
for p in pbn do
 ind:=ind*(1+(1/p));
end for;
area:=(1/3)*ind/(2^(#pbn));
return area;
end function;//of area_ind
////////////////////////////////////////////////////////////
