////////////////////////////////////////////////////////////
// Check that computed elliptic and parabolic classes
// agree with original computation. 
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// load data files.
////////////////////////////////////////////////////////////
load "datafiles/header.m";
load "datafiles/TheBigList";
load "datafiles/label_list.dat";

load "datafiles/f_groups_0";
L := L_0;
load "datafiles/f_groups_1";
L cat:= L_1;

// change to check genus 1.
load "output/fund_0.dat";
////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////
// need for matrix simplification functions.
////////////////////////////////////////////////////////////
load "psimp.m";
load "intersect.m";
////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////
// create list for conversion from f value
// to index value of squarefree list L.
////////////////////////////////////////////////////////////
convert_f_list:=[ ];
for i:=1 to #L do
 convert_f_list[L[i]`f]:=i;
end for;
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// compute the projective order of a matrix, or return -1
// if the order cannot be computed or is more than 100.
//
// if parabolic elements then return 0;
////////////////////////////////////////////////////////////
projective_matrix_order := function(m1);

order := 1;
mat := normalize_matrix(m1);
trace := mat[1]+mat[4];
det := mat[1]*mat[4] - mat[2]*mat[3];

if (trace eq 2 or trace eq -2) and det eq 1 then
 return 0;
end if;

for i := 1 to 100 do
 if mat eq [-1,0,0,-1] or mat eq [1,0,0,1] then 
  return order;
 end if;
 mat := normalize_matrix(prod(m1,mat));
 order +:=1;
end for;

print "error in projective_matrix_order, unable to compute projective order";

return -1;

end function;//of projective_matrix_order;
////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////
// main loop
////////////////////////////////////////////////////////////

for k := 1 to #fund_data do

 i := fund_data[k,1];
 j := fund_data[k,2];
 // ramification_elements := [ x[2] : x in fund_data[k,9] | x ne [1,0,0,1]];
 // change for new data
 ramification_elements := [ x[2] : x in fund_data[k,9] | x[2] ne [1,0,0,1]];
 level := fund_data[k,3];
 label := fund_data[k,4];
 genus := fund_data[k,5];
 f := fund_data[k,6];


 // compute original ramification
 original_ramification := TheBigList[i,j]`ramification;
 // change for genus 1
 top_ramification := L[convert_f_list[f]]`ramification_classes;
 if #original_ramification ne #top_ramification then 
  print "error in check_ramification, number of class not equal";
 end if;
 original_ramification_data := {* *};
 for w := 1 to #top_ramification do
  order := projective_matrix_order(top_ramification[w]);
  if order eq 0 then
   for v := 1 to #original_ramification[w] do
    original_ramification_data join:= {* 0 *};
   end for;
  else
   for v := 1 to #original_ramification[w] do
    if (order/original_ramification[w,v]) ne 1 then
     original_ramification_data join:= {* (order/original_ramification[w,v]) *};
    end if;
   end for;
  end if;
 end for;

 computed_ramification_data := {* projective_matrix_order(x) : x in ramification_elements *};

 if original_ramification_data ne computed_ramification_data then
  print "label,original_ramification_data,computed_ramification_data =",level,label,genus,f,original_ramification_data,computed_ramification_data; 
 end if;

end for;





////////////////////////////////////////////////////////////
