////////////////////////////////////////////////////////////
// cir_to_mat tries to find a transformation
// matrix m, with isometric circle
// cir which maps the points p1 and p2 which
// lie on the isometric circle into the
// interval [u1,u2]. It is assumed that
// u2-u1 is a positive integers - usually
// will be the width of the cusp at infinity.
// Used to find edge pairings of fundamental
// domain
////////////////////////////////////////////////////////////
cir_to_mat:=function(u1,u2,p1,p2,cir,g);

uwidth:=u2-u1;
in_group:=make_in_group(g);
gw:=width(g);

if not(on(p1,cir) and on(p2,cir)) then
 print "Error in cir_to_mat, point(s) not on circle";
 return -1;
else
 c:=cir[1];
 d:=cir[2];
 e:=cir[3];
 gc,pars:=ExtendedGreatestCommonDivisor([c,e*d]);
 if gc ne 1 then 
  print "Error in cir_to_mat, gc ne 1";
  return -1;
 end if;
 newmat:=[e*pars[2],-pars[1],e*c,e*d];
 np:=gact(newmat,p1);
 nx:=np[1];
 t_initial:=Floor( (nx-u1)/uwidth );
 testmats:=[x: x in [[newmat[1]-(t_initial-t)*newmat[3],newmat[2]-(t_initial-t)*newmat[4],newmat[3],newmat[4]]:
           t in [0..gw-1]] | in_group(x)];
 if #testmats eq 0 then 
  print "error, no elements in testmats in cir_to_mat";return [ ];
 elif #testmats gt 1 then 
  print "error, too many elements in testmats in cir_to_mat";return [ ];
 else 
  newmat2:=testmats[1];
 end if;
 np:=gact(newmat2,p1);
 nx:=np[1];
 if not( nx ge u1 and nx le u2) then 
  print "Error in cir_to_mat, x coordinate of transformed p1 not in [u1,u2]";
  return newmat2;
 end if;
 np2:=gact(newmat2,p2);
 nx2:=np2[1];
 if nx2 lt u1 then
  newmat3:=[newmat2[1]+uwidth*newmat2[3],newmat2[2]+uwidth*newmat2[4],newmat2[3],newmat2[4]];
 elif nx2 gt u2 then
  newmat3:=[newmat2[1]-uwidth*newmat2[3],newmat2[2]-uwidth*newmat2[4],newmat2[3],newmat2[4]];
 else
  newmat3:=newmat2;
 end if;
 return newmat3;
end if;
end function;//of cir_to_mat
////////////////////////////////////////////////////////////
