////////////////////////////////////////////////////////////
// This file computes all the fundamental domain
// data for the groups in TheBigList. This includes
// bounding circles, edge pairings, generators and ramification
// matrices.
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// load data
////////////////////////////////////////////////////////////
load "datafiles/header.m";
load "datafiles/TheBigList";
load "datafiles/label_list.dat";
load "datafiles/SL_generators_1-1000";
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// load functions
////////////////////////////////////////////////////////////
load "local_csgf.m";
load "setup.m";
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// set the genus, either 0 or 1;
// and name the data file.
////////////////////////////////////////////////////////////
computing_genus := 0;
current_date := Pipe("date","");
Data_file := "output/fund_" cat IntegerToString(computing_genus)  cat ".dat";
print "The data file is", Data_file;
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// initialize data file
////////////////////////////////////////////////////////////
fprintf Data_file,"%o%o\n","// The current date is ",current_date;
fprintf Data_file,"%o\n\n","// This file contains data from compute_data.m";
fprintf Data_file,"%o\n"," ";
fprintf Data_file,"%o\n","// The format of fund_data is:";

  fprintf Data_file,"%o\n","//[i,j,g`level,label_list[i,j],g`genus,f,g`index,";
  fprintf Data_file,"%o\n","// The i,j labels are as in TheBigList and the level,label genus and f";
  fprintf Data_file,"%o\n\n","// labels give the corresponding entry in Table 2 of the paper.";

  fprintf Data_file,"%o\n","//cover : the bounding circles of the fundamental domain. ";
  fprintf Data_file,"%o\n","//The notation is that [c,d,e] is a circle with";
  fprintf Data_file,"%o\n\n","//center -d/c and radius 1/(c*sqrt(e))";

  fprintf Data_file,"%o\n","//ram_classes : parabolic and elliptic elements fixing (some of) the";
  fprintf Data_file,"%o\n\n","//vertices of the fundamental domain.";

  fprintf Data_file,"%o\n","//pairs : the pairing of the sides of the fundamental domain by the group";
  fprintf Data_file,"%o\n\n","//the number starts at 1 with the imaginary axis and then counterclockwise.";

  fprintf Data_file,"%o\n","//pairing_mats : the matrices of the group which pair the sides.";
  fprintf Data_file,"%o\n\n","//Note that these generate the group.";

  fprintf Data_file,"%o\n","//intersection_matrices : matrices generated by inversions in the two" ;
  fprintf Data_file,"%o\n","//circles intersecting in a vertex. Note: it is sometimes necessary to" ;
  fprintf Data_file,"%o\n","//include a circles twice and include its highest point as a vertex" ;
  fprintf Data_file,"%o\n","//even if it is not an elliptic point. Then the intersection" ;
  fprintf Data_file,"%o\n\n","//matrix is taken to be [1,0,0,1]." ;

  fprintf Data_file,"%o\n","//intersection_points : the intersections of the bounding circles." ;
  fprintf Data_file,"%o\n","//The notation is that [x,y1,y2] is the point x1 + i*y1*(-y2)^(1/2)" ;
  fprintf Data_file,"%o\n\n","//with y2 squarefree." ;

  fprintf Data_file,"%o\n","//v : the orbits of the vertices under the group.";
  fprintf Data_file,"%o\n","//The labeling starts at 1 with the intersection of the imaginary";
  fprintf Data_file,"%o\n\n\n","//axis with the first circles and then counterclockwise.";

fprintf Data_file,"%o\n\n","fund_data:= [ ];";
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// main loop
////////////////////////////////////////////////////////////

max_i :=#TheBigList;
for i:=1 to max_i do
 max_j :=#TheBigList[i];
 f := TheBigList[i,1]`f;
 top_area := area_ind(f);
 print "f = ",f;
 for j:=1 to max_j do
  //if (j mod 50) eq 0 then print "j = ",j; end if;
  print "j = ",j;
  g:=TheBigList[i,j];
  if g`genus eq computing_genus then
   g_index := g`index;
   u1:=0;
   u2:=width(g);
   ga := top_area*g_index; 
 
   rc:=initial_cover(u1,u2,g,ga);
   rc:=correct_cover(u1,u2,rc,g);
   ram_classes:=ramification_classes(u1,u2,rc,g);
   gen:=genus(u1,u2,rc,g);
   pairs,pairing_mats,intersection_matrices,intersection_points:=pairing(u1,u2,rc,g);
   v:=vorbits(pairs);
   a:=area(u1,u2,rc,g);
  
   if (g`genus ne gen) or (a ne ga) then
    print "Error, g`genus,gen,a,ga = ",g`genus,gen,a,ga;
   end if;
 
   fprintf Data_file,"%o\n","Append(~fund_data,";
   fprintf Data_file,"[* %o,%o,   %o,%m,%o,%o,   %o,\n\n",i,j,g`level,label_list[i,j],g`genus,f,g`index;
   fprintf Data_file,"%o,\n",rc;
   fprintf Data_file,"%o,\n",ram_classes;
   fprintf Data_file,"%o,\n",pairs;
   fprintf Data_file,"%o,\n",pairing_mats;
   fprintf Data_file,"%o,\n",intersection_matrices;
   fprintf Data_file,"%o,\n",intersection_points;
   fprintf Data_file,"%o\n",v;
   fprintf Data_file,"%o\n\n", "*]);";
  end if;
 end for;
end for;

////////////////////////////////////////////////////////////

