////////////////////////////////////////////////////////////
// Gamma(n) example
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// load functions
////////////////////////////////////////////////////////////
load "quad.m";
load "psimp.m";
load "ptest.m";
load "interior.m";  
load "intersect.m";
load "order_tests.m";
load "epsplot2.m";
load "vintersect.m";
load "pcircles.m";  
load "fastselect.m";
load "refinecover.m";
load "int_angle.m";
load "gact.m";
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// define data structures
////////////////////////////////////////////////////////////
CongSubGrpEx := recformat<
                      f:Integers(),
                      fullmatgens:SeqEnum,//from pairing of sides of fund domain     
                      ramification_classes:SeqEnum,  
                      level:Integers(), 
                      genus:Integers(),
                      index:Integers(),
		      area:Rationals(),//area of fund domain divided by Pi
                      minus_one:GrpPermElt,
		      u1:Integers(),//left bound of fundamental domain
		      u2:Integers(),//right bound of fundamental domain
		      bounding_circles:SeqEnum,//bounding circles of fundamental domain
		      pairing:SeqEnum,//pairing of bounding cirlcles by group
		      vertex_orbits:SeqEnum//orbits of vertices, excluding infinity
		      >;
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// Gamma(n) example. Returns a function which tests if
// a given matrix is in Gamma(n)
////////////////////////////////////////////////////////////
make_in_group:=function(g);
local_in_group:=function(mat) 
if assigned g`level then n:=g`level;
if (mat[2] mod n) eq 0 and
    (mat[3] mod n) eq 0 and
    ( ( ((mat[1] -1) mod n eq  0 ) and ((mat[4] -1) mod n eq  0)  ) or 
      ( ((mat[1] +1) mod n eq  0 ) and ((mat[4] +1) mod n eq  0)  ) ) then
  return true;
 else
  return false;
 end if;
end if;
return false;
end function;

return local_in_group;
end function;//of make_in_group
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// find the width at infinity of the group g
////////////////////////////////////////////////////////////
width:=function(g);
in_group:=make_in_group(g);
t:=[1,1,0,1];
w:=1;
if in_group(t) then found:=true;else found:=false;end if;
while not(found) do
 t:=prod([1,1,0,1],t);
 w:=w+1;
 found:=in_group(t);
end while;
return w;
end function;//of width
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// Tests to see if an isometric circle is an isometric
// circle of g.
////////////////////////////////////////////////////////////
cde_in_group:=function(g,cde);
c:=cde[1];
d:=cde[2];
e:=cde[3];
in_group:=make_in_group(g);
gcd,s:=ExtendedGreatestCommonDivisor([d*e,-c]);
if gcd ne 1 then print "error in cde_in_group, gcd is not 1, it's ",gcd;end if;
mat:=[s[1]*e,s[2],c*e,d*e];
tr:=[1,1,0,1];
w:=width(g);
found:=in_group(mat);
k:=0;
while k lt w-1 and not(found) do
 mat:=prod(tr,mat);
 found:=in_group(mat);
 k:=k+1;
 end while;
return found;
end function;//of cde_in_group; 
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// return a function to determine if a circle corresponds
// to a group element.
////////////////////////////////////////////////////////////
make_cde_in_group:=function(g);

local_cde_in_group:=function(cde);
c:=cde[1];
d:=cde[2];
e:=cde[3];
in_group:=make_in_group(g);
gcd,s:=ExtendedGreatestCommonDivisor([d*e,-c]);
if gcd ne 1 then print "error in cde_in_group, gcd is not 1, it's ",gcd;end if;
mat:=[s[1]*e,s[2],c*e,d*e];
tr:=[1,1,0,1];
w:=width(g);
found:=in_group(mat);
k:=0;
while k lt w-1 and not(found) do
 mat:=prod(tr,mat);
 found:=in_group(mat);
 k:=k+1;
 end while;
return found;
end function;//of local_cde_in_group

return local_cde_in_group;

end function;//of make_cde_in_group;
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// index of Gamma(n) in PSL(2.Z)
////////////////////////////////////////////////////////////
gamman_index := function(n);

if n eq 1 then
 return 1;
end if;

if n eq 2 then
 return 6;
end if;

ans := n^3;
for p in PrimeBasis(n) do
 ans := ans*(1-(1/p^2));
end for;

return Integers()!(ans/2);

end function;//of gamman_index
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// more loading
////////////////////////////////////////////////////////////
load "cirtomat.m";
load "pairs.m";
load "angle.m";
load "info.m";
load "genus.m"; 
load "genusf.m";
load "area.m";
load "initialcover.m";
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// main loop
////////////////////////////////////////////////////////////

for n:=2 to 11 do

 print "n = ",n;

 g:=rec<CongSubGrpEx | f:=1, level:=n>;
 ga := gamman_index(n)/3;
 u1:=0;
 u2:=width(g);
 rc := initial_cover(u1,u2,g,ga);
 ram_classes:=ramification_classes(u1,u2,rc,g);
 gen := genus(u1,u2,rc,g);
 geometric_area := area(u1,u2,rc,g);


 print "bounding circles =",rc;
 print "ramification classes = ",ram_classes;
 print "genus =",gen;
 print "geometric area = ",geometric_area;
 print "area from index = ",ga;
 print " ";

end for;

////////////////////////////////////////////////////////////
