////////////////////////////////////////////////////////////
// functions for computing genus of Gamma_0(f)^+ i.e.
// Gamma_0(f) with all Atkin-Lehner involutions (f squarefree).
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// compute extended Legendre symbol
// See Shimura p25
////////////////////////////////////////////////////////////
LS:=function(a,p);

if a eq -4 or a eq -1 then
 if p eq 2 then return(0); end if;
 if ( (p-1) mod 4) eq 0 then return(1); end if;
 if ( (p-3) mod 4) eq 0 then return(-1); end if;
end if;

if a eq -3 then
 if p eq 3 then return(0); end if;
 if ( (p-1) mod 3) eq 0 then return(1); end if;
 if ( (p-2) mod 3) eq 0 then return(-1); end if;
end if;

print "Error in LS";
return(-99);

end function;
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// Compute genus of \Gamma_0(f)^+ using
// Helling's formula
////////////////////////////////////////////////////////////
genusf:=function(f)

if f eq 1 then return 0; end if;

if not(IsSquarefree(f)) then 
 print "Error: f must be square-free in genusf";
 return(-1);
end if;

F:=[k[1] : k in Factorization(f)];
pif:=#F;

mi:=1;
m2:=1;
m3:=1;
for p in F do
mi:=mi*(1+p);
m2:=m2*(1+LS(-4,p));
m3:=m3*(1+LS(-3,p));
end for;



g0:=( 12*(1 - 2^(pif-1)) +mi -3*m2 -4*m3) div 12;

W:=0;

if (f mod 2) eq 1 then

for mD in Divisors(4*f) do
D:=-mD;
if ( (D mod 4) eq 0 or ((D-1) mod 4) eq 0) and (D ne -4) then
t:=1;
for p in F do
 t:=t*(1+LegendreSymbol(D,p));
end for;
W:=W + ClassNumber(BinaryQuadraticForms(D))*t;
end if;
end for;
 

else

W0:=0;
W1:=0;
for mD in Divisors(4*f) do
D:=-mD;

if ( (D mod 4) eq 0 ) or (D eq -3) then
t:=1;
for p in F do
 if p ne 2 then t:=t*(1+LegendreSymbol(D,p)); end if;
end for;
W0:=W0 + ClassNumber(BinaryQuadraticForms(D))*t;
end if;

if ( (D mod 4) eq 1 ) and (D ne -3) then
t:=1;
for p in F do
 if p ne 2 then t:=t*(1+LegendreSymbol(D,p)); end if;
end for;
W1:=W1 + 3*ClassNumber(BinaryQuadraticForms(D))*t;
end if;

end for;
W:=W0+W1;
 
end if;

g:=1+(( g0-1 - (W div 2) ) div 2^(pif));
return(g);
end function;//of genusf
////////////////////////////////////////////////////////////
