////////////////////////////////////////////////////////////
// define data structures
////////////////////////////////////////////////////////////
CongSubGrpEx := recformat<
                      f:Integers(),
                      fullmatgens:SeqEnum,//from pairing of sides of fund domain     
                      ramification_classes:SeqEnum,  
                      level:Integers(), 
                      genus:Integers(),
                      index:Integers(),
		      area:Rationals(),//area of fund domain divided by Pi
                      minus_one:GrpPermElt,
		      u1:Integers(),//left bound of fundamental domain
		      u2:Integers(),//right bound of fundamental domain
		      bounding_circles:SeqEnum,//bounding circles of fundamental domain
		      pairing:SeqEnum,//pairing of bounding cirlcles by group
		      vertex_orbits:SeqEnum//orbits of vertices, excluding infinity
		      >;
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// make a procedure to test if the matrix [a,b,c,d] is in
// the group g. The data structure is as in TheBigList which
// is data from the computation of the genus zero and one
// congruence groups.
////////////////////////////////////////////////////////////
make_in_group:=function(g);
local_in_group:=function(mat) 
n := g`level;
f := g`f;
localgroup := g`localgroup;
a:=mat[1];b:=mat[2];c:=mat[3];d:=mat[4];
e := a*d-b*c;
if not(IsDivisibleBy(c,f)) or not(IsDivisibleBy(a,e)) or not(IsDivisibleBy(d,e)) then
 print "Error, matrix is not in Gamma_0(f)^+ in local_in_group, f,mat = ",f,mat;
end if;
perm := Sym(Degree(localgroup))!perm_n([a,b,c,d],f,n);
if perm in localgroup then
 return true;
else
 return false;
end if;
end function;

return local_in_group;
end function;//of make_in_group
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// find the width at infinity of the group g
////////////////////////////////////////////////////////////
width:=function(g);
in_group:=make_in_group(g);
t:=[1,1,0,1];
w:=1;
if in_group(t) then found:=true;else found:=false;end if;
while not(found) do
 t:=prod([1,1,0,1],t);
 w:=w+1;
 found:=in_group(t);
end while;
return w;
end function;//of width
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// make a function to test if a circle corresponds to a 
// group element.
////////////////////////////////////////////////////////////
cde_in_group:=function(g,cde);
c:=cde[1];
d:=cde[2];
e:=cde[3];
in_group:=make_in_group(g);
gcd,s:=ExtendedGreatestCommonDivisor([d*e,-c]);
if gcd ne 1 then print "error in cde_in_group, gcd is not 1, it's ",gcd;end if;
mat:=[s[1]*e,s[2],c*e,d*e];
tr:=[1,1,0,1];
w:=width(g);
found:=in_group(mat);
k:=0;
while k lt w-1 and not(found) do
 mat:=prod(tr,mat);
 found:=in_group(mat);
 k:=k+1;
 end while;
return found;
end function;//of cde_in_group;
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// makes a function to test if a circle corresponds to
// a group element.
////////////////////////////////////////////////////////////
make_cde_in_group:=function(g);

local_cde_in_group:=function(cde);
c:=cde[1];
d:=cde[2];
e:=cde[3];
in_group:=make_in_group(g);
gcd,s:=ExtendedGreatestCommonDivisor([d*e,-c]);
if gcd ne 1 then print "error in cde_in_group, gcd is not 1, it's ",gcd;end if;
mat:=[s[1]*e,s[2],c*e,d*e];
tr:=[1,1,0,1];
w:=width(g);
found:=in_group(mat);
k:=0;
while k lt w-1 and not(found) do
 mat:=prod(tr,mat);
 found:=in_group(mat);
 k:=k+1;
 end while;
return found;
end function;//of local_cde_in_group

return local_cde_in_group;

end function;//of make_cde_in_group;
////////////////////////////////////////////////////////////
