////////////////////////////////////////////////////////////
// main procedure for computing the fundamental domain
// of a subgroup of Gamma_0(f)^+
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// computes a cover of
// of the interval [u1,u2] by
// circles corresponding to  subgroup of
// Gamma_0(f)^+.
// Computes all circles of radius 
// greater than or equal to
// 1/k and then checks to see if the area is computable.
// If so, this is checked to see if it
// equal to group_area. If not, then k is
// incremented. This must terminate, since there
// is a cover of [u1,u2] which gives a fundamental domain
// for which the areas must be equal.
////////////////////////////////////////////////////////////
initial_cover:=function(u1,u2,g,group_area);

if group_area le 0 then
 print "Error in initial_cover, group_area must be positive, group_area = ",group_area;
end if;

// can change inc. No theory for optimal choice.
inc:=Floor(g`f/4)+1;
k:=-inc;

cde_in_group:=make_cde_in_group(g);

C:=[ ];

// set up a non-empty cover to start.
// note: now we are using the area
// criterion, we don't have to test
// that is covers the interval [u1,u2]
// since if it doesn't then the area
// computation fails and we refine the cover
NC:=[ ];
while #NC eq 0 do
 k:=k+inc;
 // make some new circles
 L(k,k+inc,u1,u2,~g,~cde_in_group,~NC);
end while;
refine_cover(u1,u2,~C,~NC);


working:=true;
while working do
 // note if area fails to compute the area, then
 // -1 is returned, so geometric_area eq group_area
 // is false (since group_area is positive).
 geometric_area:=area(u1,u2,C,g);
 print "working... geometric area = ",geometric_area;
 if geometric_area eq group_area then
  working := false;
 end if;
 if working then
  NC:=[ ];
  while #NC eq 0 do
   k:=k+inc;
   // make some new circles
   L(k,k+inc,u1,u2,~g,~cde_in_group,~NC);
  end while;
  refine_cover(u1,u2,~C,~NC);
 end if;
end while;
 
return C;

end function;//of initial_cover
////////////////////////////////////////////////////////////
