////////////////////////////////////////////////////////////
// computes  the sine and cosine
// of the angle of intersection of
// two circles of form
// [c,d,e] with centre -d/c and
// radius 1/(e^(1/2)c)
////////////////////////////////////////////////////////////
int_angle:=function(cir1,cir2);

// special case: if the two circles are equal then
// by convention they intersect at an angle of pi 
// at the highest point
if cir1 eq cir2 then
 return [-1],[1],[0],[1];
end if;

c1:=cir1[1];
d1:=cir1[2];
e1:=cir1[3];
c2:=cir2[1];
d2:=cir2[2];
e2:=cir2[3];

if -d1/c1 gt -d2/c2 then
 print "warning cir1 to right of cir2, exchanging...";
 c1:=cir2[1];
 d1:=cir2[2];
 e1:=cir2[3];
 c2:=cir1[1];
 d2:=cir1[2];
 e2:=cir1[3];
end if;

int_true,p:=intersect(cir1,cir2);

if not(int_true) then
 print "Error, circles do not intersect in int_order";
 return -1;
else
 x:=p[1];
 num:=(((d1/c1) - (d2/c2))*x +(d1/c1)^2 - (d1*d2)/(c1*c2) - 1/(c1^2*e1));
 sig:=Sign(num);
 cos_sq_angle:=num^2*( c1^2*e1*c2^2*e2 );
 sin_sq_angle:=1-cos_sq_angle;
end if;

if cos_sq_angle eq 0 then
 c1:=[Rationals()!0];q1:=[Rationals()!1];
else
 c1,q1:=quadsimp([Rationals()!sig],[Rationals()!cos_sq_angle]);
end if;
if sin_sq_angle eq 0 then
 s1:=[Rationals()!0];e1:=[Rationals()!1];
else
 s1,e1:=quadsimp([Rationals()!1],[Rationals()!sin_sq_angle]);
end if;

return c1,q1,s1,e1;

end function;//of int_angle
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// same as above, but finds the intersection
// is with the line x=u. Take orientation
// 1 for the left boundary and -1 for
// the right boundary
////////////////////////////////////////////////////////////
line_angle:=function(cir1,u,orientation);

c1:=cir1[1];
d1:=cir1[2];
e1:=cir1[3];

int_true,p:=vintersect(cir1,u);

if not(int_true) then
 print "Error, line does not intersect circle in line_true";
 return -1;
else
 x:=p[1];
 num:=-orientation*(x+(d1/c1));
 sig:=Sign(num);
 cos_sq_angle:=num^2*(c1^2*e1);
 sin_sq_angle:=1-cos_sq_angle;
end if;

if cos_sq_angle eq 0 then
 c1:=[Rationals()!0];q1:=[Rationals()!1];
else
 c1,q1:=quadsimp([Rationals()!sig],[Rationals()!cos_sq_angle]);
end if;
if sin_sq_angle eq 0 then
 s1:=[Rationals()!0];e1:=[Rationals()!1];
else
 s1,e1:=quadsimp([Rationals()!1],[Rationals()!sin_sq_angle]);
end if;

return c1,q1,s1,e1;
  
end function;//of line_angle
////////////////////////////////////////////////////////////
