////////////////////////////////////////////////////////////
// functions to test relative position of a circle and
// a point.
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// test to see if point p is in the interior of circle cir.
// p:=[x,r,q] corresponds to x + y*sqrt(r)
// cir:=[c,d,e] is circle with centre -d/c
// and radius 1/(c*sqrt(e))
////////////////////////////////////////////////////////////
interior:=function(p,cir);

c:=cir[1];
d:=cir[2];
e:=cir[3];
ps:=psimp(p);
x:=ps[1];
y:=ps[2];
r:=ps[3];

//bug  in magma
if r in Integers() then r:=Integers()!r; end if;

if c eq 0 then print "error, c eq 0 in interior";return -1;end if;

if r gt 0 and y ne 0 then 
 
// print "r =",r,"Is integer r",r in Integers();
 mytest:= ptest( (x+(d/c))^2+y^2*r-1/(c^2*e),2*y*(x+(d/c)),0,r,1);
 if mytest eq -1 then
  return true;
 else
  return false;
 end if;
end if;

if (x+d/c)^2 - y^2*r lt 1/(c^2*e) then
 return true;
else
 return false;
end if;

end function;//of interior
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// test to see if point p is in interior to or on  circle cir.
// p:=[x,y,r]] corresponds to x + y*sqrt(r)
// cir:=[c,d,e] is circle with centre -d/c
// and radius 1/(c*sqrt(e))
////////////////////////////////////////////////////////////

interior_or_on:=function(p,cir);

c:=cir[1];
d:=cir[2];
e:=cir[3];
ps:=psimp(p);
x:=ps[1];
y:=ps[2];
r:=ps[3];

//bug  in magma
if r in Integers() then r:=Integers()!r; end if;

if c eq 0 then print "error, c eq 0 in interior";return -1;end if;

if r gt 0 and y ne 0 then 
 mytest:= ptest( (x+(d/c))^2+y^2*r-1/(c^2*e),2*y*(x+(d/c)),0,r,1);
 if mytest eq -1 or mytest eq 0 then
  return true;
 else
  return false;
 end if;
end if;

if (x+d/c)^2 - y^2*r le 1/(c^2*e) then
 return true;
else
 return false;
end if;

end function;//of interior_or_on
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// same as above, but now on the circle.
////////////////////////////////////////////////////////////
on:=function(p,cir);

c:=cir[1];
d:=cir[2];
e:=cir[3];
ps:=psimp(p);
x:=ps[1];
y:=ps[2];
r:=ps[3];

//bug  in magma
if r in Integers() then r:=Integers()!r; end if;

if c eq 0 then print "error, c eq 0 in interior";return -1;end if;

if r gt 0 and y ne 0 then 
 mytest:= ptest( (x+(d/c))^2+y^2*r-1/(c^2*e),2*y*(x+(d/c)),0,r,1);
 if mytest eq 0 then
  return true;
 else
  return false;
 end if;
end if;

if (x+d/c)^2 - y^2*r eq 1/(c^2*e) then
 return true;
else
 return false;
end if;

end function;//of on
////////////////////////////////////////////////////////////


