////////////////////////////////////////////////////////////
// This file defines just those functions needed to
// compute fundamental domains. These are a subset
// of the functions originally used to compute the genus
// zero and genus one congruence subgroups. 
////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////
// given 0<=c< 2^(pi(f))-1 encode as binary and compute
// the corresponding factor of the square free integer f
////////////////////////////////////////////////////////////
convert_c_to_e:=function(c,f);
F:=Factorization(f);
lc:=c;
ans:=1;
for i:=1 to #F do
ans:=ans*F[i][1]^(lc mod 2);
lc:=lc div 2;
end for;
return ans;
end function;//of convert_c_to_e
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// inverse of convert_c_to_e
////////////////////////////////////////////////////////////
convert_e_to_c:=function(e,f);
F:=Factorization(f);
ans:=0;
for i:=1 to #F do
if (Integers()!e mod Integers()!F[#F-i+1][1]) eq 0 then ans:=2*ans+1; else ans:=2*ans; end if;
end for;
return ans;
end function;//of convert_e_to_c
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// given a matrix m in SL(2,Z) and a representation
// of S and T, compute the representation of m
//
// recall S=[0,1,-1,0], T=[1,1,0,1], I=[1,0,0,1]
////////////////////////////////////////////////////////////
sl2z_word:=function(m,S,T,I);

a:=Integers()!m[1];
b:=Integers()!m[2];
c:=Integers()!m[3];
d:=Integers()!m[4];

ans:=I;

done:=false;
while not done do
 if a eq 1 and b eq 0 and c eq 0 and d eq 1 then
  done:=true;
 elif c eq 0 then
  if a lt 0 then
   a:=-a;
   b:=-b;
   d:=-d;
   ans:=ans*S^2;
  else
   done:=true;
   ans:=ans*T^b;
  end if;
 elif c^2 gt a^2 then
  na:=c; nb:=d; nc:=-a; nd:=-b;
  a:=na; b:=nb; c:=nc; d:=nd;
  ans:=ans*S^3;
 else
  q:=Quotrem(a,c);
  a:=a-q*c;
  b:=b-q*d;
  ans:=ans*T^q;
 end if;
end while;

return ans;

end function;//of sl2z_word
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// compute permutation action of m=[a,b,c,d] \in \Gamma_0(n)^+
// new version uses precomputed permutation representations
// of SL(2,Z/p^nZ)
//
// Note: must first load generators for representations of SL(2,Z/p^nZ)
// from previously computed data.
// Note: changed from one input matrix to
// list of matrices for efficiency (only
// have to create overgroup once).
// not currently used in perm_n
//
// permutation representation of m
// in group \Gamma(f)^+/G(f,p^N)
////////////////////////////////////////////////////////////
perm:=function(list_of_matrices,f,p,N);

n:=p^N;

// recall in SL(2,Z)
// S=[0,1,-1,0] and T=[1,1,0,1]
S_deg:=SL2_primepower_generators[n][1];
S:=SL2_primepower_generators[n][2];
T:=SL2_primepower_generators[n][3];
MS:=[0,1,-1,0];
MT:=[1,1,0,1];

G:=sub<Sym(S_deg) | S, T>;

//Note f is square free
F:=Factorization(f);
//number of prime factors of f
pif:=#F;
// degree of representation
deg:=2^(pif)*S_deg;

Answer:=[ ];

//repeat for each matrix in list
for m in list_of_matrices do

 //ans is will be final permutation
 ans:=[ ];


 //entries of matrix m
 a:=Integers()!m[1];
 b:=Integers()!m[2];
 c:=Integers()!m[3];
 d:=Integers()!m[4];
 if GreatestCommonDivisor([a,b,c,d]) ne 1 then print "WARNING in perm, gcd of matrix entries is not 1"; end if;
 // reduced determinant of m
 mb:=a*d-b*c;

 for ci:=0 to (2^pif)-1 do
  e:=convert_c_to_e(ci,f);
  g:=GreatestCommonDivisor(e,mb);
  new_e:=Integers()!((e*mb)/g^2) ;
  new_ci:=convert_e_to_c(new_e,f);
//  SL2_mat:=[ Integers()!((a*g)/mb) , Integers()!((b*f*g)/(e*mb)) , Integers()!((c*e)/(f*g)), Integers()!(d/g) ];
// changed so that action is from the right. See paper
  SL2_mat:=[ Integers()!(a/g) , Integers()!((b*f*g)/(e*mb)) , Integers()!((c*e)/(f*g)), Integers()!((d*g)/mb) ];
  perm_list:=[j+S_deg*new_ci : j in ElementToSequence(sl2z_word(SL2_mat,G.1,G.2,G.0))];
  ans cat:=perm_list;
 end for;

 Append(~Answer,ans);
end for;

return Answer;
end function;//of perm
////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////
// compute image of matrix m in permutation representation of
// \Gamma_0(f)^+/G(f,n) as computed by perm
// returns permutation as a list.
////////////////////////////////////////////////////////////
perm_n := function(m,f,n)

if n eq 1 then factL:=[<1,1>]; else factL := Factorization(n);end if;

M := 0;
S := [ ];
for fact in factL do
 //note: not currently using multiple matrices in perm,
 //so select first matrix returned
 s:=perm([m],f,fact[1],fact[2])[1];
 ss := [ u+M : u in s ];
 S cat:= ss;
 M := M + #s;
end for;

return S;
end function;//of perm_n
////////////////////////////////////////////////////////////
      
