////////////////////////////////////////////////////////////
// functions to test that the projective order of elliptic
// and parabolic elements are consistent with computed angle
// at vertex.
////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////
// compute the projective order of a matrix, or return -1
// if the order cannot be computed or is more than 100.
////////////////////////////////////////////////////////////
projective_matrix_order := function(m1);

order := 1;
mat := m1;
for i := 1 to 100 do
 if mat eq [-1,0,0,-1] or mat eq [1,0,0,1] then 
  return order;
 end if;
 mat := normalize_matrix(prod(m1,mat));
 order +:=1;
end for;

print "error in projective_matrix_order, unable to compute projective order";

return -1;

end function;//of projective_matrix_order;
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// test to see if parabolic or elliptic elements are
// consistent with computed angle. Note angle has
// factor of Pi removed.
////////////////////////////////////////////////////////////
test_order_and_angle := function(mat,angle);
 
 mat := normalize_matrix(mat);
 trace := mat[1] + mat[4];
 det := mat[1]*mat[4] - mat[2]*mat[3];
 
 // test if matrix is 1 or -1
 if mat eq [1,0,0,1] then
  if angle eq 2 then return true; else return false; end if;
 end if;
 if mat eq [-1,0,0,-1] then
  if angle eq 2 then return true; else return false; end if;
 end if;

 // test if matrix if parabolic
 if det eq 1 and trace eq 2 and angle eq 0 then return true; end if;
 if det eq 1 and trace eq -2 and angle eq 0 then return true; end if;

 // matrix should be elliptic, so compute the order and test.
 order := projective_matrix_order(mat);
 if order lt 0 then
  print "error in test_order_and_angle, cannot compute order of matrix"; 
  return -1; 
 end if;

 if order eq 2/angle then
  return true;
 else
  return false;
 end if;

end function;//test_order_and_angle
////////////////////////////////////////////////////////////
