////////////////////////////////////////////////////////////
// L(k1,k2,u1,u2,f) returns a list consisting of
// entries [c,d,e] such that for some a,b, g=[[a*e,b],[c*e,d*e]]
// is an element of Gamma_0(f)^+ such that the isometric
// circle of g has center v and radius r with u1<= v <= u2
// and k1 < 1/r <= k2.
// note that v = -d/c and
// r = 1/(e^(1/2)*c)
// 
// This is used in initialcover.m to added new bounding circles
// those bounding the fundamental domain.
////////////////////////////////////////////////////////////
L:=procedure(k1,k2,u1,u2,~g,~cde_in_group,~ans);
for e in Divisors(g`f) do
 for c:=0 to k2 do
  if GreatestCommonDivisor(c,e) eq 1 and (c*e mod g`f) eq 0 then
   if (e*c^2 gt k1^2) and (e*c^2 le k2^2) then
    for md:=c*u1 to c*u2 do
     if GreatestCommonDivisor(c,md) eq 1 then
      if cde_in_group([c,-md,e]) then
       Append(~ans,[c,-md,e]);
      end if;
     end if;
    end for;
   end if;
  end if;
 end for;
end for;
end procedure;//of L
////////////////////////////////////////////////////////////



     
