////////////////////////////////////////////////////////////
// Determines if cir intersects the vertical line x=a
// in the upper half plane. If
// so, returns coords of intersection point
// in the upper half plane and the product of
// the inversions in cir and x=a (which
// fixes the intersection point). The form
// of cir is [c,d,e] for a circle with
// center -d/c and radius 1/(c*sqrt(e))
////////////////////////////////////////////////////////////
vintersect:=function(cir,a);

// initialize variables
c1:=cir[1];
d1:=cir[2];
e1:=cir[3];

// Inversions represented by matrices.
// Note: there is also a complex conjugation
// to be applied with each matrix, but as
// we are taking product of two inversions
// these cancel.
inv1:=[-d1*c1*e1, 1-d1^2*e1, c1^2*e1, c1*d1*e1];
inv2:=[-1, 2*a, 0, 1];

m:=prod(inv1,inv2);

a:=Integers()!m[1];
b:=Integers()!m[2];
c:=Integers()!m[3];
d:=Integers()!m[4];

gcd:=GreatestCommonDivisor([a,b,c,d]);

a:=Integers()!(a/gcd);
b:=Integers()!(b/gcd);
c:=Integers()!(c/gcd);
d:=Integers()!(d/gcd);

product_mat:=[a,b,c,d];

if c eq 0 then 
 if d-a ne 0 then
  intersects:=true;
  xi:=b/(d-a);
  yi:=[0,1];
 else
  if b eq 0 then
   print "warning, intersection point is not unique";
   intersects:=true;
   xi:=-d1/c1;
   yi:=[1/(c1*e1),-e1];
  else
   intersects:=false;
  end if;
 end if;
else //c ne 0
 rad:=(d-a)^2+4*b*c;
 if rad gt 0 then
  intersects:=false;
 else
  intersects:=true;
  xi:=(a-d)/(2*c);
  if rad eq 0 then
   yi:=[0,1];
  else
   p,q:=SquareFree(rad);
   yi:=[AbsoluteValue(q/(2*c)),p];
  end if;
 end if;
end if;
 
if intersects then 
 point:=psimp([xi,yi[1],yi[2]]);
 return true,point,product_mat ;
else
 return false, _, _  ;
end if;

end function;//of vintersect
////////////////////////////////////////////////////////////
