////////////////////////////////////////////////////////////
// Non-congruence subgroup example
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// Define a permutation representation of SL(2,Z) 
//
// recall S=[0,1,-1,0], T=[1,1,0,1], I = [1,0,0,1];
//
// Note: user must check that this really is a representation
// of SL(2,Z);
//
// Change this to compute other examples.
////////////////////////////////////////////////////////////

G := Sym(7);
S := G!(2,4)(3,6)(5,7);
T := G!(1,2,5,7,3)(4,6);

print "S*T = ",S*T;

I := Id(G);
////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////
// load functions
////////////////////////////////////////////////////////////
load "quad.m";
load "psimp.m";
load "ptest.m";
load "interior.m";  
load "intersect.m";
load "order_tests.m";
load "epsplot2.m";
load "vintersect.m";
load "pcircles.m";  
load "fastselect.m";
load "refinecover.m";
load "int_angle.m";
load "gact.m";
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// define data structures
////////////////////////////////////////////////////////////
CongSubGrpEx := recformat<
                      f:Integers(),
                      fullmatgens:SeqEnum,//from pairing of sides of fund domain     
                      ramification_classes:SeqEnum,  
                      level:Integers(), 
                      genus:Integers(),
                      index:Integers(),
		      area:Rationals(),//area of fund domain divided by Pi
                      minus_one:GrpPermElt,
		      u1:Integers(),//left bound of fundamental domain
		      u2:Integers(),//right bound of fundamental domain
		      bounding_circles:SeqEnum,//bounding circles of fundamental domain
		      pairing:SeqEnum,//pairing of bounding cirlcles by group
		      vertex_orbits:SeqEnum//orbits of vertices, excluding infinity
		      >;
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
//given a matrix m in SL(2,Z) and a representation
//of S and T, compute the representation of m
//
//recall S=[0,1,-1,0], T=[1,1,0,1], I=[1,0,0,1]
////////////////////////////////////////////////////////////
sl2z_word:=function(m,S,T,I);

a:=Integers()!m[1];
b:=Integers()!m[2];
c:=Integers()!m[3];
d:=Integers()!m[4];

ans:=I;

done:=false;
while not done do
 if a eq 1 and b eq 0 and c eq 0 and d eq 1 then
  done:=true;
 elif c eq 0 then
  if a lt 0 then
   a:=-a;
   b:=-b;
   d:=-d;
   ans:=ans*S^2;
  else
   done:=true;
   ans:=ans*T^b;
  end if;
 elif c^2 gt a^2 then
  na:=c; nb:=d; nc:=-a; nd:=-b;
  a:=na; b:=nb; c:=nc; d:=nd;
  ans:=ans*S^3;
 else
  q:=Quotrem(a,c);
  a:=a-q*c;
  b:=b-q*d;
  ans:=ans*T^q;
 end if;
end while;

return ans;

end function;//of sl2z_word
////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////
// Decision procedure for the stabilizer of 1 
// of the permutation representation defined above
////////////////////////////////////////////////////////////
make_in_group:=function(g);

local_in_group:=function(mat) 
 word:=sl2z_word(mat,S,T,I);
 if 1^word eq 1 then
   return true;
  else
   return false;
 end if;
end function;

return local_in_group;
end function;//of make_in_group
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// find the width at infinity of the group g
////////////////////////////////////////////////////////////
width:=function(g);
in_group:=make_in_group(g);
t:=[1,1,0,1];
w:=1;
if in_group(t) then found:=true;else found:=false;end if;
while not(found) do
 t:=prod([1,1,0,1],t);
 w:=w+1;
 found:=in_group(t);
end while;
return w;
end function;//of width
////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////
// Does the isometric circle defined by the triple c,d,e
// correspond to a group element?
////////////////////////////////////////////////////////////
cde_in_group:=function(g,cde);
c:=cde[1];
d:=cde[2];
e:=cde[3];
in_group:=make_in_group(g);
gcd,s:=ExtendedGreatestCommonDivisor([d*e,-c]);
if gcd ne 1 then print "error in cde_in_group, gcd is not 1, it's ",gcd;end if;
mat:=[s[1]*e,s[2],c*e,d*e];
tr:=[1,1,0,1];
w:=width(g);
found:=in_group(mat);
k:=0;
while k lt w-1 and not(found) do
 mat:=prod(tr,mat);
 found:=in_group(mat);
 k:=k+1;
 end while;
return found;
end function;//of cde_in_group; 
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// return a function to determine if a circle corresponds
// to a group element.
////////////////////////////////////////////////////////////
make_cde_in_group:=function(g);

local_cde_in_group:=function(cde);
c:=cde[1];
d:=cde[2];
e:=cde[3];
in_group:=make_in_group(g);
gcd,s:=ExtendedGreatestCommonDivisor([d*e,-c]);
if gcd ne 1 then print "error in cde_in_group, gcd is not 1, it's ",gcd;end if;
mat:=[s[1]*e,s[2],c*e,d*e];
tr:=[1,1,0,1];
w:=width(g);
found:=in_group(mat);
k:=0;
while k lt w-1 and not(found) do
 mat:=prod(tr,mat);
 found:=in_group(mat);
 k:=k+1;
 end while;
return found;
end function;//of local_cde_in_group

return local_cde_in_group;

end function;//of make_cde_in_group;
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// more loading
////////////////////////////////////////////////////////////
load "cirtomat.m";
load "pairs.m";
load "angle.m";
load "info.m";
load "genus.m"; 
load "genusf.m";
load "area.m";
load "initialcover.m";
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// main loop
////////////////////////////////////////////////////////////

 // define g as a record of type CongSubGrpEx
 g:=rec<CongSubGrpEx |f:=1 >;
 // compute the area of the fundamental domain in multiples of Pi
 // which is the index in SL(2,Z) divided by 3 or the Degree of
 // the premutation representation divided by 3.
 ga := Degree(G)/3;
 //left and right bounds of the fundamental domain
 u1:=0;
 u2:=width(g);
 print "width =",u2;

 // compute the bounding circles of the fundamental domain
 rc := initial_cover(u1,u2,g,ga);
 rc := correct_cover(u1,u2,rc,g);
 // compute the ramification classes of g from the fundamental domain
 ram_classes:=ramification_classes(u1,u2,rc,g);
 // compute the genus from the fundamental domain
 gen := genus(u1,u2,rc,g);
 // compute the remaining data as explained in the paper
 pairs,pairing_mats,intersection_matrices,intersection_points:=pairing(u1,u2,rc,g);
 v:=vorbits(pairs);
 geometric_area := area(u1,u2,rc,g);

 // print the results
 print "bounding circles = ",rc;
 print "ramification classes = ",ram_classes;
 print "genus = ",gen;
 print "geometric area = ",geometric_area;
 print "area from index = ",ga;
 print "pairing = ",pairs;
 print "pairing_matrices = ",pairing_mats;
 print "vertex orbits = ",v;
 print " ";

////////////////////////////////////////////////////////////
